"use strict";

var _express = _interopRequireWildcard(require("express"));

var _bodyParser = _interopRequireDefault(require("body-parser"));

var _mongoose = _interopRequireDefault(require("mongoose"));

var _supertest = _interopRequireDefault(require("supertest"));

var _ = _interopRequireDefault(require(".."));

var _jwt = require("../lib/jwt");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; if (obj != null) { var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

let id;
let secret;
let jwt;
let nonce;
let apiKey;
const app = (0, _express.default)();
const router = (0, _express.Router)();
const request = (0, _supertest.default)(app);
const host = 'localhost';
const role = 'Admin';

const withHeaders = req => req.set('Accept', 'application/json').set('Authorization', `Bearer ${jwt}`).set('exp-nonce', nonce).set('host', host);

beforeAll(async () => {
  const auth = (0, _.default)(_mongoose.default);
  router.get('/', (req, res) => {
    res.status(200).send(res.marshal({
      firstName: 'Jane',
      lastName: 'Doe',
      gender: 'Female',
      age: 21
    }));
  }).post('/', auth.middleware.sanitize, (req, res) => {
    res.status(201).json(req.body);
  }).delete('/', (req, res) => {
    res.status(204).send();
  });
  app.use(_bodyParser.default.json());
  app.use(auth.middleware.init);
  app.use('/foo', auth.middleware.define('e-auth-users'), router);
  const {
    Access,
    User
  } = auth.models;
  const doc = await User.create({
    firstName: 'Jon',
    lastName: 'Doe',
    email: 'jon.doe@gmail.com',
    password: 'Hey!',
    secret: 'secret',
    active: true,
    verified: true,
    role
  });
  ({
    _id: id,
    secret
  } = doc);
  await doc.setPassword();
  apiKey = await doc.generateApiKey();
  await Access.create([{
    op: 'Read',
    coll: 'e-auth-users',
    fields: '(.*name*)',
    role
  }, {
    op: 'Create',
    coll: 'e-auth-users',
    fields: 'firstName',
    role
  }]);
});
beforeEach(async () => {
  process.env.SECRET = 'shh!';
  ({
    token: jwt,
    nonce
  } = await (0, _jwt.generateIDToken)(id, secret, host));
});
describe('Foo', () => {
  it('should reject unauthorized requests', async () => {
    await withHeaders(request.delete('/foo')).expect(403);
  });
  it('should filter returned data', async () => {
    const {
      body
    } = await withHeaders(request.get('/foo')).expect(200);
    expect(body).toEqual({
      firstName: 'Jane',
      lastName: 'Doe'
    });
  });
  it('should filter body query/payload', async () => {
    const {
      body
    } = await withHeaders(request.post('/foo').send({
      firstName: 'Jon',
      lastName: 'Doe',
      age: 27
    })).expect(201);
    expect(body).toEqual({
      firstName: 'Jon'
    });
  });
  it('should decode API key', async () => {
    await request.get('/foo').set('Authorization', `Apikey ${apiKey}`).expect(200);
  });
});