"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.verifyToken = exports.generateIDToken = exports.default = void 0;

var _jsonwebtoken = _interopRequireDefault(require("jsonwebtoken"));

var _bcrypt = require("./bcrypt");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _default = null;
exports.default = _default;

const decode = async (token = '') => {
  return _jsonwebtoken.default.verify(token.slice(7, token.length), process.env.SECRET);
};

const generateIDToken = async (id, code, audience) => {
  if (!id) throw new Error('ID required to sign the jwt payload');
  const secret = process.env.SECRET;
  const nonce = (0, _bcrypt.generateRandomSecret)(16);
  const token = await _jsonwebtoken.default.sign({
    nonce,
    id,
    code
  }, secret, {
    audience
  });
  return {
    token,
    nonce
  };
};

exports.generateIDToken = generateIDToken;

const verifyToken = async (token, nonce, host, User) => {
  try {
    const {
      id,
      aud,
      code,
      nonce: secretNonce
    } = await decode(token);
    const user = await User.findVerifiedById(id);
    if (nonce !== secretNonce || aud !== host || !user.isPermitted || user.secret !== code) throw new Error('Invalid token context or user state');
    return user;
  } catch (err) {
    // log it?
    return null;
  }
};

exports.verifyToken = verifyToken;