"use strict";

var _mongoose = _interopRequireDefault(require("mongoose"));

var _ = _interopRequireDefault(require(".."));

var _tests = require("../../helpers/tests");

var _access = _interopRequireDefault(require("../../models/access"));

var _user = _interopRequireDefault(require("../../models/user"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let init;
let define;
const stub = 'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.SflKxwRJSMeKKF2QT4fwpMeJf36POk6yJV_adQssw5c';
const header = jest.fn().mockImplementation(name => name === 'Authorization' ? stub : null);
const next = jest.fn();
const req = {
  get: jest.fn(),
  method: 'PATCH',
  header
};
jest.mock('../../lib/jwt', () => ({
  verifyToken: jest.fn().mockResolvedValueOnce(null).mockResolvedValueOnce({
    _id: 1
  })
}));
beforeEach(() => {
  next.mockReset();
});
beforeAll(async () => {
  const User = _mongoose.default.model('mid-users', _user.default);

  const Access = _mongoose.default.model('mid-permissions', _access.default);

  await Access.create({
    coll: 'mid-permissions',
    op: 'Update',
    role: 'Administrator'
  });
  ({
    init,
    define
  } = (0, _.default)({
    Access,
    User
  }));
});
describe('init', () => {
  it('set to public without a bearer', async () => {
    await init(req, null, next);
    expect(req.user).toBeNull();
    expect(next).toHaveBeenCalled();
  });
  it('should append user details on token decode', async () => {
    await init(req, null, next);
    expect(req.user).toMatchObject({
      _id: 1
    });
  });
});
describe('define', () => {
  it('should throw an error if unauthorized', async () => {
    Object.assign(req, {
      validateSession: jest.fn().mockImplementation(async () => {
        throw new Error('Whoops...');
      })
    });
    const fn = define('events');
    await (0, _tests.willReject)(fn(req, null, next));
  });
  it('should overwrite body on authorization', async () => {
    const res = {
      status: jest.fn()
    };
    Object.assign(req, {
      method: 'PATCH',
      user: {
        role: 'Administrator'
      },
      body: {
        foo: 'bar'
      }
    });
    const fn = define('mid-permissions');
    await fn(req, res, next);
    expect(res.marshal).toEqual(expect.any(Function));
  });
});