"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _lodash = require("lodash");

var _jwt = require("../lib/jwt");

const convertHTTPtoOp = method => {
  switch (method) {
    case 'PATCH':
    case 'PUT':
      return 'Update';

    case 'GET':
      return 'Read';

    case 'POST':
      return 'Create';

    case 'DELETE':
      return 'Delete';

    default:
      throw new Error('Method not allowed');
  }
};

var _default = models => ({
  async init(req, res, next) {
    const {
      User
    } = models;
    if (!User) throw new Error('Requires User model to validate');
    req.user = await (0, _jwt.verifyToken)(req.header('Authorization'), req.header('exp-nonce'), req.get('host'), User);

    if (!req.user) {
      req.user = await User.findByApiKey(req.header('Authorization'));
    }

    next();
  },

  define(coll) {
    return async (req, res, next) => {
      const {
        Access
      } = models;

      try {
        const {
          method,
          user
        } = req;
        const getPermission = Access.can(convertHTTPtoOp(method));
        const grants = await getPermission((0, _lodash.get)(user, 'role'), coll);
        Object.assign(res, {
          marshal: grants.pickFrom ? grants.pickFrom.bind(grants) : v => v
        });
        next();
      } catch (err) {
        res.status(403);
        next(err);
      }
    };
  },

  sanitize(req, res, next) {
    Object.assign(req, {
      body: res.marshal(req.body)
    });
    next();
  }

});

exports.default = _default;