"use strict";

var _decorator = _interopRequireDefault(require("../decorator"));

var _tests = require("../../../helpers/tests");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const select = jest.fn();
const chain = {
  findOne: jest.fn().mockReturnValue({
    select
  })
};
describe('can', () => {
  it('should return falsy', async () => {
    select.mockResolvedValue(null);

    const pm = _decorator.default.can.call(chain, 'Create');

    await (0, _tests.willReject)(pm('Administrator', 'events'));
    expect(chain.findOne).toHaveBeenCalledWith({
      role: 'Administrator',
      coll: 'events',
      op: 'Create'
    });
  });
  it('should return with allowed fields', async () => {
    select.mockResolvedValue({
      fields: '(.*)'
    });

    const pm = _decorator.default.can.call(chain, 'Update');

    await (0, _tests.willResolveObject)(pm(chain), {
      fields: '(.*)'
    });
  });
});
describe('pickFrom', () => {
  it('should strip out excluded properties from object', () => {
    const inst = new _decorator.default();
    inst.fields = '(foo+), baz, waldo.quux';
    expect(inst.pickFrom({
      foo: 1,
      bar: 2,
      baz: {
        id: 3
      },
      garply: 4,
      waldo: {
        quux: 3
      },
      foobar: 6
    })).toMatchObject({
      foo: 1,
      baz: {
        id: 3
      },
      waldo: {
        quux: 3
      },
      foobar: 6
    });
  });
});