"use strict";

var _mongoose = _interopRequireDefault(require("mongoose"));

var _express = _interopRequireDefault(require("express"));

var _bodyParser = _interopRequireDefault(require("body-parser"));

var _supertest = _interopRequireDefault(require("supertest"));

var _access = _interopRequireDefault(require("../access"));

var _access2 = _interopRequireDefault(require("../../models/access"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let id;
const app = (0, _express.default)();
const request = (0, _supertest.default)(app);
const administrator = 'Admin';
const seed = [{
  role: administrator,
  op: 'Update',
  coll: 'permissions',
  fields: 'fields'
}, {
  role: administrator,
  op: 'Create',
  coll: 'permissions'
}, {
  role: administrator,
  op: 'Read',
  coll: 'permissions',
  fields: 'op, role'
}, {
  role: administrator,
  op: 'Delete',
  coll: 'permissions'
}, {
  op: 'Read',
  coll: 'permissions'
}];
beforeAll(async () => {
  const Access = _mongoose.default.model('mock-access', _access2.default);

  await Access.create(seed);
  app.use(_bodyParser.default.json());
  app.use((req, res, next) => {
    Object.assign(res, {
      marshal: v => v
    });
    next();
  });
  app.use((0, _access.default)(_mongoose.default, {
    Access
  }));
});
describe('/GET', () => {
  it('should return 200', async () => {
    const {
      body
    } = await request.get('/permissions').expect(200);
    expect(body.permissions).toHaveLength(seed.length);
    expect(body.colls).toEqual(expect.arrayContaining(['mock-access']));
    expect(body.roles).toEqual(expect.arrayContaining([administrator]));
  });
  it('should return 200 by role', async () => {
    const {
      body
    } = await request.get(`/permissions?role=${administrator}`).expect(200);
    expect(body.permissions).toHaveLength(seed.filter(({
      role
    }) => role === administrator).length);
  });
  it('should return 200 by operation', async () => {
    const {
      body
    } = await request.get('/permissions?op=Read').expect(200);
    expect(body.permissions).toHaveLength(seed.filter(({
      op
    }) => op === 'Read').length);
  });
  it('should return 200 by operation and role', async () => {
    const {
      body
    } = await request.get(`/permissions?op=Read&role=${administrator}`).expect(200);
    expect(body.permissions).toHaveLength(seed.filter(({
      role
    }) => !role).length);
  });
});
describe('/POST', () => {
  it('should return 201', async () => {
    const {
      body
    } = await request.post('/permissions').send({
      op: 'Create',
      coll: 'permissions',
      role: 'Sales'
    }).expect(201);
    expect(body.permission).toHaveProperty('id');
    ({
      id
    } = body.permission);
  });
});
describe('/PATCH', () => {
  it('should return 200', async () => {
    await request.patch(`/permissions/${id}`).send({
      fields: 'op, coll'
    }).expect(200);
  });
});
describe('/DELETE', () => {
  it('should return 200', async () => {
    await request.delete(`/permissions/${id}`).expect(204);
  });
});