"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _express = require("express");

var _lodash = require("lodash");

var _expressAsyncHandler = _interopRequireDefault(require("express-async-handler"));

var _helpers = require("../helpers");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _default = (db, Models) => {
  const router = (0, _express.Router)();
  const {
    Access
  } = Models;

  const fetchMetaData = async () => {
    const roles = await Access.distinct('role').lean();
    const colls = (0, _helpers.getAllCollections)(db);
    return {
      roles,
      colls
    };
  };

  const fetchPermissions = async ({
    query
  }, res) => {
    const {
      docs,
      ...rest
    } = await Access.paginate(query, {
      limit: 25,
      lean: true
    });
    res.status(200).json({ ...rest,
      ...(await fetchMetaData()),
      permissions: docs.map(res.marshal)
    });
  };

  const fetchById = async ({
    params
  }, res) => {
    const {
      permissionsID
    } = params;
    const doc = await Access.findById(permissionsID);

    if (!doc) {
      res.status(404).send();
    } else {
      res.status(200).json({ ...(await fetchMetaData()),
        permission: doc
      });
    }
  };

  const updateById = async ({
    body,
    params
  }, res) => {
    const {
      permissionsID
    } = params;
    const doc = await Access.findByIdAndUpdate(permissionsID, {
      fields: body.fields
    }, {
      useFindAndModify: false,
      runValidators: true,
      new: true
    });
    const permission = res.marshal(doc.toJSON({
      virtuals: true
    }));
    res.status(200).json({
      message: `Permission #${permissionsID} successfully updated`,
      permission
    });
  };

  const createPermission = async ({
    body
  }, res) => {
    const doc = await Access.create((0, _lodash.pick)(body, ['op', 'coll', 'fields', 'role']));
    const permission = res.marshal(doc.toJSON({
      virtuals: true
    }));
    res.status(201).json({
      message: `New ${doc.id} permission added to ${doc.coll}`,
      permission
    });
  };

  const deleteById = async ({
    params
  }, res) => {
    const {
      permissionsID
    } = params;
    await Access.findByIdAndDelete(permissionsID, {
      useFindAndModify: false
    });
    res.status(204).send();
  };

  router.route('/permissions').get((0, _expressAsyncHandler.default)(fetchPermissions)).post((0, _expressAsyncHandler.default)(createPermission));
  router.route('/permissions/:permissionsID').get((0, _expressAsyncHandler.default)(fetchById)).patch((0, _expressAsyncHandler.default)(updateById)).delete((0, _expressAsyncHandler.default)(deleteById));
  return router;
};

exports.default = _default;