"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _express = require("express");

var _expressAsyncHandler = _interopRequireDefault(require("express-async-handler"));

var _jwt = require("../lib/jwt");

var _helpers = require("../helpers");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _default = (db, models) => {
  const router = (0, _express.Router)();
  const {
    User,
    Access
  } = models;
  router.post('/authenticate', (0, _expressAsyncHandler.default)(async (req, res) => {
    const origin = req.get('host');
    const {
      email,
      password
    } = req.body;
    const userResult = await User.findVerifiedByEmail(email);

    if (!userResult.isPermitted) {
      res.status(403).send();
    } else if (!(await userResult.verifyPassword(password))) {
      res.status(401).send();
    } else {
      const {
        _id: id,
        secret
      } = userResult;
      const tokens = await (0, _jwt.generateIDToken)(id, secret, origin);
      res.status(201).json(tokens);
    }
  }));
  router.post('/verify', (0, _expressAsyncHandler.default)(async ({
    body
  }, res) => {
    const {
      id,
      password,
      verificationCode
    } = body;
    const doc = await User.findUserBySecret(id, verificationCode);

    if (doc.hasExpired) {
      res.status(410).send();
    } else if (!password) {
      res.status(422).json({
        password: 'Requires a password'
      });
    } else {
      await doc.setPassword(password);
      res.status(204).send();
    }
  }));
  router.post('/reverify', (0, _expressAsyncHandler.default)(async ({
    body
  }, res) => {
    const {
      email
    } = body;
    const doc = await User.findUnverifiedByEmail(email);
    await doc.setSecret();
    res.status(204).send();
  }));
  router.post('/password-reset', (0, _expressAsyncHandler.default)(async ({
    body
  }, res) => {
    const {
      email
    } = body;
    const doc = await User.findVerifiedByEmail(email);
    const password = await doc.setPassword();

    _helpers.notify.emit('password-reset', {
      email,
      password
    });

    res.status(204).send();
  }));
  router.post('/password-update', (0, _expressAsyncHandler.default)(async ({
    body,
    user
  }, res) => {
    const {
      previousPassword,
      newPassword,
      confirmNewPassword
    } = body;

    if (!user) {
      res.status(401).send();
    } else if (newPassword !== confirmNewPassword || !newPassword) {
      res.status(422).json({
        newPassword: 'Does not match confirmation password',
        confirmNewPassword: 'Does not match new password'
      });
    } else {
      const doc = await User.findVerifiedById(user.id);
      await doc.verifyPassword(previousPassword, true);
      await doc.setPassword(newPassword);
      res.status(204).send();
    }
  }));
  router.get('/profile', (0, _expressAsyncHandler.default)(async ({
    user
  }, res) => {
    if (!user) {
      res.status(401).send();
    } else {
      res.json({
        profile: user.obfuscatePrivateFields(),
        permissions: await Access.find({
          role: user.role
        }).lean()
      });
    }
  }));
  return router;
};

exports.default = _default;